<?php

namespace Drupal\Tests\snippet_manager\Functional;

use Drupal\Component\Utility\Html;

/**
 * Test for CodeMirror assets.
 *
 * @group snippet_manager
 */
class CodeMirrorAssetsTest extends TestBase {

  /**
   * Tests that CodeMirror assets are loaded properly.
   */
  public function testAssets() {

    $this->drupalGet('admin/structure/snippet/alpha/edit/template');

    $codemirror_version = '5.34.0';

    $css_files = [
      'codemirror.min.css',
      'addon/fold/foldgutter.min.css',
      'addon/display/fullscreen.css',
    ];

    $js_files = [
      'codemirror.min.js',
      'mode/javascript/javascript.min.js',
      'mode/xml/xml.min.js',
      'mode/twig/twig.min.js',
      'mode/css/css.min.js',
      'addon/fold/foldcode.min.js',
      'addon/fold/brace-fold.min.js',
      'addon/fold/xml-fold.min.js',
      'addon/fold/comment-fold.min.js',
      'addon/display/fullscreen.min.js',
    ];

    // Check CDN mode.
    foreach ($css_files as $file) {
      $this->assertXpath('//head/link[@rel="stylesheet" and @href="https://cdnjs.cloudflare.com/ajax/libs/codemirror/' . $codemirror_version . '/' . $file . '"]');
    }

    foreach ($js_files as $file) {
      $this->assertXpath('//body/script[@src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/' . $codemirror_version . '/' . $file . '"]');
    }

    // Check local mode and non-default theme.
    $edit = [
      'codemirror[cdn]' => FALSE,
    ];
    $this->drupalPostForm('admin/structure/snippet/settings', $edit, 'Save configuration');

    $this->drupalGet('admin/structure/snippet/alpha/edit/template');

    $imports = explode("\n", $this->xpath('//head/style')[0]->getHtml());
    $query_string = \Drupal::state()->get('system.css_js_query_string');

    // TODO: Test theme files.
    foreach ($css_files as $file) {
      $file = 'libraries/codemirror/' . $file;
      $import = '@import url("' . Html::escape(file_url_transform_relative(file_create_url($file)) . '?' . $query_string) . '");';
      $this->assertTrue(in_array($import, $imports));
    }

    foreach ($js_files as $file) {
      $file = 'libraries/codemirror/' . $file . '?v=' . $codemirror_version;
      $file = file_url_transform_relative(file_create_url($file));
      $this->assertXpath('//body/script[@src="' . $file . '"]');
    }

  }

}
